part of katana_routing;

/// Define a variable container.
@immutable
abstract class Container<TResult> {
  const Container();
  ContainerState<TResult, Container<TResult>> createContainer();
}

/// State created from Container.
abstract class ContainerState<TResult, TContainer extends Container<TResult>> {
  WidgetRef? _ref;
  TContainer? _container;

  /// Current container.
  TContainer get container => _container!;

  /// When executed, [Scoped], [ScopedWidget] and [PageScopedWidget] will be rebuilt.
  ///
  /// [callback] is called before the rebuild is done.
  @protected
  @mustCallSuper
  void setState(VoidCallback callback) {
    callback.call();
    _ref?.rebuild();
  }

  /// Process when the container is updated.
  @mustCallSuper
  void didUpdateContainer(covariant TContainer oldContainer) {}

  /// Called when the container is created.
  @mustCallSuper
  void initContainer() {}

  /// Called when the page is Disposed.
  @mustCallSuper
  void dispose() {}

  /// Build and output the value.
  TResult build();
}
